@@ -2,36 +2,25 @@ from __future__ import print_function
 from __future__ import absolute_import
 from __future__ import division
 
-from ast import literal_eval
+import json
 
-import compas
 import compas_rhino
 
+import System
+import Rhino
+import rhinoscriptsyntax as rs
+import scriptcontext as sc
+
+find_object = sc.doc.Objects.Find
+
 try:
-    import System
-    import Rhino
-    import rhinoscriptsyntax as rs
-    import scriptcontext as sc
-
-    find_object = sc.doc.Objects.Find
-
-except ImportError:
-    compas.raise_if_ironpython()
-
-else:
-    try:
-        purge_object = sc.doc.Objects.Purge
-    except AttributeError:
-        purge_object = None
-
-
-# get_point_coordinates => get_coordinates_points
-# get_line_coordinates => get_coordinates_lines
-# get_polyline_coordinates => get_coordinates_polylines
-# get_polygon_coordinates => get_coordinates_polygons
+    purge_object = sc.doc.Objects.Purge
+except AttributeError:
+    purge_object = None
 
 
 __all__ = [
+    'find_object',
     'get_objects',
     'get_object_layers',
     'get_object_types',
@@ -41,7 +30,6 @@ __all__ = [
     'get_object_attributes_from_name',
     'delete_object',
     'delete_objects',
-    'delete_objects_by_name',
     'purge_objects',
     'is_curve_line',
     'is_curve_polyline',
@@ -87,8 +75,61 @@ __all__ = [
 # Objects
 # ==============================================================================
 
-
 def get_objects(name=None, color=None, layer=None, type=None):
+    """Get identifiers of Rhino objects, potentially filtered by name, color, layer, or type.
+
+    Parameters
+    ----------
+    name : str, optional
+    color : tuple or list, optional
+        RGB color components in integer format (0-255).
+    layer : str, optional
+    type : Rhino.DocObjects.ObjectType, optional
+        The object type.
+
+    Returns
+    -------
+    list
+        The GUIDs of the objects matching the filter parameters.
+
+    Examples
+    --------
+    .. code-block:: python
+
+        import compas_rhino
+
+        guids_all = compas_rhino.get_objects()
+        guids_compas = compas_rhino.get_objects(name='COMPAS.*')
+        guids_red = compas_rhino.get_objects(color=(255, 0, 0))
+        guids_points = compas_rhino.get_objects(type=compas_rhino.rs.filter.point)
+        guids_redpoints = compas_rhino.get_objects(color=(255, 0, 0), type=compas_rhino.rs.filter.point)
+
+    .. code-block:: python
+
+        guids_all = set(compas_rhino.get_objects())
+        guids_compas = set(compas_rhino.get_objects(name='COMPAS.*'))
+        guids_red = set(compas_rhino.get_objects(color=(255, 0, 0)))
+        guids_points = set(compas_rhino.get_objects(type=compas_rhino.rs.filter.point))
+        guids_redpoints = set(compas_rhino.get_objects(color=(255, 0, 0), type=compas_rhino.rs.filter.point))
+
+        print guids_compas.issubset(guids_all)
+        print guids_all.issubset(guids_compas)
+
+        # True, False
+
+        print guids_red.issubset(guids_all)
+        print guids_points.issubset(guids_all)
+        print guids_redpoints.issubset(guids_all)
+
+        # True, True, True
+
+        print guids_redpoints.issubset(guids_points)
+        print guids_redpoints.issubset(guids_red)
+        print guids_points.issubset(guids_red)
+
+        # True, True, False
+
+    """
     guids = rs.AllObjects()
     if name:
         guids = list(set(guids) & set(rs.ObjectsByName(name)))
@@ -97,14 +138,24 @@ def get_objects(name=None, color=None, layer=None, type=None):
     if layer:
         guids = list(set(guids) & set(rs.ObjectsByLayer(layer)))
     if type:
-        guids = list(set(guids) & set(rs))
+        guids = list(set(guids) & set(rs.ObjectsByType(type)))
     return guids
 
 
 def delete_object(guid, purge=None):
+    """Delete Rhino object.
+
+    Parameters
+    ----------
+    guid : GUID
+    purge : None or bool, optional
+        If None, the value of the global purge setting (:obj:`compas_rhino.PURGE_ON_DELETE`) will be used.
+        If True, purge the object from history after deleting.
+        If False, delete but don't purge.
+        Default is None.
+    """
     if purge is None:
         purge = compas_rhino.PURGE_ON_DELETE
-
     if purge and purge_object:
         purge_objects([guid])
     else:
@@ -112,9 +163,19 @@ def delete_object(guid, purge=None):
 
 
 def delete_objects(guids, purge=None):
+    """Delete multiple Rhino objects.
+
+    Parameters
+    ----------
+    guids : list of GUID
+    purge : None or bool, optional
+        If None, the value of the global purge setting (:obj:`compas_rhino.PURGE_ON_DELETE`) will be used.
+        If True, purge the objects from history after deleting.
+        If False, delete but don't purge.
+        Default is None.
+    """
     if purge is None:
         purge = compas_rhino.PURGE_ON_DELETE
-
     if purge and purge_object:
         purge_objects(guids)
     else:
@@ -127,15 +188,13 @@ def delete_objects(guids, purge=None):
         sc.doc.Views.Redraw()
 
 
-def delete_objects_by_name(name, purge=None):
-    if purge is None:
-        purge = compas_rhino.PURGE_ON_DELETE
-
-    guids = get_objects(name)
-    delete_objects(guids, purge=purge)
-
-
 def purge_objects(guids):
+    """Purge objects from memory.
+
+    Parameters
+    ----------
+    guids : list of GUID
+    """
     if not purge_object:
         raise RuntimeError('Cannot purge outside Rhino script context')
     rs.EnableRedraw(False)
@@ -150,6 +209,16 @@ def purge_objects(guids):
 
 
 def get_object_layers(guids):
+    """Get the layer names of multiple objects.
+
+    Parameters
+    ----------
+    guids : list of GUID
+
+    Returns
+    -------
+
+    """
     return [rs.ObjectLayer(guid) for guid in guids]
 
 
@@ -166,6 +235,16 @@ def get_object_name(guid):
 
 
 def get_object_attributes(guids):
+    """Get attributes from object user dictionaries.
+
+    Parameters
+    ----------
+    guids : list of GUID
+
+    Returns
+    -------
+    list of dict
+    """
     attrs = []
     for guid in guids:
         o = find_object(guid)
@@ -179,12 +258,28 @@ def get_object_attributes(guids):
 
 
 def get_object_attributes_from_name(guids, prefix=None):
+    """Get attributes from JSON parsable object names.
+
+    Parameters
+    ----------
+    guids : list of GUID
+    prefix : str, optional
+        A prefix that should be removed before the name is JSON parsable.
+        For example, in Rhino 6 and above, names can't start with curly braces ("{").
+        To make the string representation of a dict a valid name it has to be prefixed with something like "COMPAS_".
+
+    Results
+    -------
+    list of dict
+        The attribute dicts of the objects.
+    """
+    load = json.loads
     attrs = []
     for name in get_object_names(guids):
         try:
             if prefix:
                 name = name.lstrip(prefix)
-            attr = literal_eval(name)
+            attr = load(name)
         except (ValueError, TypeError):
             attr = {}
         attrs.append(attr)
@@ -192,10 +287,34 @@ def get_object_attributes_from_name(guids, prefix=None):
 
 
 def select_object(message="Select an object."):
+    """Select one object in the Rhino view.
+
+    Parameters
+    ----------
+    message : str, optional
+        Default is "Select an object.".
+
+    Returns
+    -------
+    GUID
+        The identifer of the selected object.
+    """
     return rs.GetObject(message)
 
 
-def select_objects(message='Select objects.'):
+def select_objects(message='Select multiple objects.'):
+    """Select multiple objects in the Rhino view.
+
+    Parameters
+    ----------
+    message : str, optional
+        Default is "Select objects.".
+
+    Returns
+    -------
+    list of GUID
+        The identifers of the selected objects.
+    """
     guids = []
     temp = rs.GetObjects(message)
     if temp:
@@ -208,13 +327,37 @@ def select_objects(message='Select objects.'):
 # ==============================================================================
 
 
-def select_point(message='Select a point.'):
-    return rs.GetObject(message, filter=rs.filter.point)
+def select_point(message='Select one point.'):
+    """Select one point in the Rhino view.
+
+    Parameters
+    ----------
+    message : str, optional
+        Default is "Select an point.".
+
+    Returns
+    -------
+    GUID
+        The identifer of the selected point.
+    """
+    return rs.GetObject(message, preselect=True, select=True, group=False, filter=rs.filter.point)
 
 
-def select_points(message='Select points.'):
+def select_points(message='Select multiple points.'):
+    """Select multiple points in the Rhino view.
+
+    Parameters
+    ----------
+    message : str, optional
+        Default is "Select points.".
+
+    Returns
+    -------
+    list of GUID
+        The identifers of the selected points.
+    """
     guids = []
-    temp = rs.GetObjects(message, filter=rs.filter.point)
+    temp = rs.GetObjects(message, preselect=True, select=True, group=False, filter=rs.filter.point)
     if temp:
         guids = temp
     return guids
@@ -236,6 +379,17 @@ def get_points(layer=None):
 
 
 def get_point_coordinates(guids):
+    """Get the coordintes of the locations of point objects.
+
+    Parameters
+    ----------
+    guids : list of GUID
+
+    Returns
+    -------
+    list of point
+        The location coordinates of the points.
+    """
     points = []
     for guid in guids:
         point = rs.PointCoordinates(guid)
@@ -261,11 +415,35 @@ def is_curve_polygon(guid):
     return rs.IsCurve(guid) and rs.IsCurveClosed(guid) and rs.CurveDegree(guid) == 1 and len(rs.CurvePoints(guid)) > 2
 
 
-def select_curve(message='Select curve.'):
-    return rs.GetObject(message, filter=rs.filter.curve)
+def select_curve(message='Select one curve.'):
+    """Select one curve in the Rhino view.
+
+    Parameters
+    ----------
+    message : str, optional
+        Default is "Select an line.".
+
+    Returns
+    -------
+    GUID
+        The identifer of the selected curve.
+    """
+    return rs.GetObject(message, preselect=True, select=True, group=False, filter=rs.filter.curve)
 
 
-def select_curves(message='Select curves.'):
+def select_curves(message='Select multiple curves.'):
+    """Select multiple curves in the Rhino view.
+
+    Parameters
+    ----------
+    message : str, optional
+        Default is "Select an line.".
+
+    Returns
+    -------
+    list of GUID
+        The identifers of the selected curves.
+    """
     guids = []
     temp = rs.GetObjects(message, filter=rs.filter.curve)
     if temp:
@@ -274,15 +452,39 @@ def select_curves(message='Select curves.'):
 
 
 def select_line(message='Select line.'):
-    guid = rs.GetObject(message, filter=rs.filter.curve)
+    """Select one line in the Rhino view.
+
+    Parameters
+    ----------
+    message : str, optional
+        Default is "Select an line.".
+
+    Returns
+    -------
+    GUID
+        The identifer of the selected line.
+    """
+    guid = rs.GetObject(message, preselect=True, select=True, group=False, filter=rs.filter.curve)
     if is_curve_line(guid):
         return guid
     return None
 
 
-def select_lines(message='Select lines.'):
+def select_lines(message='Select multiple lines.'):
+    """Select multiple lines in the Rhino view.
+
+    Parameters
+    ----------
+    message : str, optional
+        Default is "Select lines.".
+
+    Returns
+    -------
+    list of GUID
+        The identifers of the selected lines.
+    """
     guids = []
-    temp = rs.GetObjects(message, filter=rs.filter.curve)
+    temp = rs.GetObjects(message, preselect=True, select=True, group=False, filter=rs.filter.curve)
     if temp:
         for guid in temp:
             if is_curve_line(guid):
@@ -290,16 +492,40 @@ def select_lines(message='Select lines.'):
     return guids
 
 
-def select_polyline(message='Select a polyline (curve with degree = 1, and multiple segments).'):
+def select_polyline(message='Select one polyline (curve with degree = 1, and multiple segments).'):
+    """Select one polyline in the Rhino view.
+
+    Parameters
+    ----------
+    message : str, optional
+        Default is "Select an polyline.".
+
+    Returns
+    -------
+    GUID
+        The identifer of the selected polyline.
+    """
     guid = rs.GetObject(message, filter=rs.filter.curve)
     if is_curve_polyline(guid):
         return guid
     return None
 
 
-def select_polylines(message='Select polylines (curves with degree = 1, and multiple segments).'):
+def select_polylines(message='Select multiple polylines (curves with degree = 1, and multiple segments).'):
+    """Select multiple polylines in the Rhino view.
+
+    Parameters
+    ----------
+    message : str, optional
+        Default is "Select polylines.".
+
+    Returns
+    -------
+    list of GUID
+        The identifers of the selected polylines.
+    """
     guids = []
-    temp = rs.GetObjects(message, filter=rs.filter.curve)
+    temp = rs.GetObjects(message, preselect=True, select=True, group=False, filter=rs.filter.curve)
     if temp:
         for guid in temp:
             if is_curve_polyline(guid):
@@ -307,16 +533,40 @@ def select_polylines(message='Select polylines (curves with degree = 1, and mult
     return guids
 
 
-def select_polygon(message='Select a polygon (closed curve with degree = 1)'):
-    guid = rs.GetObject(message, filter=rs.filter.curve)
+def select_polygon(message='Select one polygon (closed curve with degree = 1)'):
+    """Select one polygon in the Rhino view.
+
+    Parameters
+    ----------
+    message : str, optional
+        Default is "Select an polygon.".
+
+    Returns
+    -------
+    GUID
+        The identifer of the selected polygon.
+    """
+    guid = rs.GetObject(message, preselect=True, select=True, group=False, filter=rs.filter.curve)
     if is_curve_polygon(guid):
         return guid
     return None
 
 
-def select_polygons(message='Select polygons (closed curves with degree = 1)'):
+def select_polygons(message='Select multiple polygons (closed curves with degree = 1)'):
+    """Select multiple polygons in the Rhino view.
+
+    Parameters
+    ----------
+    message : str, optional
+        Default is "Select polygons.".
+
+    Returns
+    -------
+    list of GUID
+        The identifers of the selected polygons.
+    """
     guids = []
-    temp = rs.GetObjects(message, filter=rs.filter.curve)
+    temp = rs.GetObjects(message, preselect=True, select=True, group=False, filter=rs.filter.curve)
     if temp:
         for guid in temp:
             if is_curve_polygon(guid):
@@ -451,14 +701,38 @@ def get_polygon_coordinates(guids):
 # ==============================================================================
 
 
-def select_surface(message='Select a surface.'):
+def select_surface(message='Select one surface.'):
+    """Select one surface in the Rhino view.
+
+    Parameters
+    ----------
+    message : str, optional
+        Default is "Select an surface.".
+
+    Returns
+    -------
+    GUID
+        The identifer of the selected surface.
+    """
     return rs.GetObject(
         message,
         filter=rs.filter.surface | rs.filter.polysurface
     )
 
 
-def select_surfaces(message='Select surfaces.'):
+def select_surfaces(message='Select multiple surfaces.'):
+    """Select multiple surfaces in the Rhino view.
+
+    Parameters
+    ----------
+    message : str, optional
+        Default is "Select surfaces.".
+
+    Returns
+    -------
+    list of GUID
+        The identifers of the selected surfaces.
+    """
     guids = []
     temp = rs.GetObjects(message, filter=rs.filter.surface | rs.filter.polysurface)
     if temp:
@@ -471,14 +745,38 @@ def select_surfaces(message='Select surfaces.'):
 # ==============================================================================
 
 
-def select_mesh(message='Select a mesh.'):
+def select_mesh(message='Select one mesh.'):
+    """Select one mesh in the Rhino view.
+
+    Parameters
+    ----------
+    message : str, optional
+        Default is "Select an mesh.".
+
+    Returns
+    -------
+    GUID
+        The identifer of the selected mesh.
+    """
     return rs.GetObject(
         message,
         filter=rs.filter.mesh
     )
 
 
-def select_meshes(message='Select meshes.'):
+def select_meshes(message='Select multiple meshes.'):
+    """Select multiple meshes in the Rhino view.
+
+    Parameters
+    ----------
+    message : str, optional
+        Default is "Select meshs.".
+
+    Returns
+    -------
+    list of GUID
+        The identifers of the selected meshs.
+    """
     guids = []
     temp = rs.GetObjects(message, filter=rs.filter.mesh)
     if temp:
@@ -548,7 +846,7 @@ def get_mesh_vertex_index(guid):
         def CustomGeometryFilter(self, rhino_object, geometry, component_index):
             return guid == rhino_object.Id
     go = CustomGetObject()
-    go.SetCommandPrompt('Select a vertex of the mesh.')
+    go.SetCommandPrompt('Select one vertex of the mesh.')
     go.GeometryFilter = Rhino.DocObjects.ObjectType.MeshVertex
     go.AcceptNothing(True)
     if go.Get() != Rhino.Input.GetResult.Object:
@@ -570,7 +868,7 @@ def get_mesh_face_index(guid):
         def CustomGeometryFilter(self, rhino_object, geometry, component_index):
             return guid == rhino_object.Id
     go = CustomGetObject()
-    go.SetCommandPrompt('Select a face of the mesh.')
+    go.SetCommandPrompt('Select one face of the mesh.')
     go.GeometryFilter = Rhino.DocObjects.ObjectType.MeshFace
     go.AcceptNothing(True)
     if go.Get() != Rhino.Input.GetResult.Object:
